import click
import os
from pathlib import Path
from .constants import sites, languages
from .utils import list_tolower


class Generator:
    def __init__(self, PATH, GIT, CONFIG):
        self.PATH = PATH
        self.GIT = GIT
        self.CONFIG = CONFIG

    def generate(self):
        i, CURR_PROBLEM, CURR_SITE = 0, "", ""
        for subdir, dirs, files in os.walk(self.PATH, topdown=True):
            # Exclude ignored directories
            dirs[:] = self.exclude_dirs(subdir, dirs)

            # Create READMEs
            for d in dirs:
                if d.lower() in list_tolower(self.CONFIG['Paths']):
                    try:
                        self.create_readme(os.path.join(subdir, d))
                    except Exception as err:
                        print("Error trying to create READMEs.", err)

            for file in files:
                if self.check_subdir(subdir):
                    PATH = os.path.join(subdir, file)
                    # Check if at a solution file
                    if os.path.split(os.path.split(PATH)[0])[1].lower() not in list_tolower(self.CONFIG['Paths']):
                        filename, file_extension = os.path.splitext(
                            PATH)
                        problem_name = os.path.split(os.path.split(
                            PATH)[0])[1].lower()
                        file_dir = os.path.dirname(subdir)
                        site_name = os.path.basename(file_dir).lower()
                        path_parts = Path(PATH).parts
                        print(os.path.basename(os.path.normpath(self.PATH)))
                        git_path = self.GIT + "/blob/master/" + \
                            os.path.join(
                                *path_parts[len(path_parts)-path_parts.index(os.path.basename(os.path.normpath(self.PATH)))+1:])

                        if CURR_SITE != site_name:
                            i = 0

                        if site_name not in sites:
                            print("Invalid folder name.\n Exiting...")
                            exit()

                        if CURR_PROBLEM != problem_name:
                            print(
                                "[~] Creating table entry for: {}/{}...".format(site_name, problem_name))
                            readme = open("{}/README.md".format(file_dir), "a")
                            if i > 0:
                                readme.write("\n")
                            readme.write(" [{}]({}/{}) | [{}]({})".format(
                                problem_name.capitalize(), sites[site_name], problem_name, languages[file_extension], git_path))
                        else:
                            print(
                                "[~] Appending to existing table entry for {}/{}".format(site_name, problem_name))
                            readme = open(
                                "{}/README.md".format(file_dir), "a")
                            readme.write(", [{}]({}) ".format(
                                languages[file_extension], git_path).rstrip())

                        CURR_PROBLEM, CURR_SITE = problem_name, site_name
                        i += 1
                        readme.close()

        click.secho("Process finished!", bold=True)

    def exclude_dirs(self, subdir, dirs):
        self.CONFIG['Ignore'], self.CONFIG['Paths'] = list_tolower(
            self.CONFIG['Ignore']), list_tolower(self.CONFIG['Paths'])
        return [d for d in dirs if d.lower() in self.CONFIG['Paths'] and d.lower() not in self.CONFIG['Ignore'] or self.check_subdir(subdir)]

    def check_subdir(self, subdir):
        subdir = subdir.lower()
        for i in list_tolower(self.CONFIG['Paths']):
            if i in subdir:
                return True
        return False

    def create_readme(self, path):
        title = os.path.basename(path)
        print("[~] Creating README for {}...".format(title))
        with open(os.path.join(path, "README.md"), "w") as f:
            f.write("# {}\n".format(title))
            f.write("| Problem | Languages |\n")
            f.write("| ------- | --------- |\n")
            f.close()



from sorcerer.utils import get_slice_length


def test_get_slice_length():
    paths = {
        "../../example": 6,
        "../example": 3,
        "../../../example": 9
    }
    for k, v in paths.items():
        assert(get_slice_length(k) == v)



import click
import json
import os
from .generator import Generator


@click.command()
@click.option('--git', prompt='Github link to repository')
@click.option('--local_path', prompt='Local path to repository')
def cli(git, local_path):
    # Load json config
    CONFIG = {}
    if os.path.exists('{}/config.json'.format(local_path)):
        with open('{}/config.json'.format(local_path)) as f:
            try:
                CONFIG = json.load(f)
            except Exception as e:
                print(e)
                exit()
    else:
        print("Config file not found.")
        exit()

    generator = Generator(local_path, git, CONFIG)
    generator.generate()


if __name__ == '__main__':
    cli()



from sorcerer.utils import list_tolower


def test_list_tolower():
    tests = [
        (["AbC", "aBc", "abc"], ["abc", "abc", "abc"]),
        (["XyZ", "OOOO", "zZzZz"], ["xyz", "oooo", "zzzzz"]),
        (["A", "b", "c", "D"], ["a", "b", "c", "d"])
    ]
    for k, v in tests:
        assert(list_tolower(k) == v)



sites = {
    "kattis": "https://open.kattis.com/problems",
    "leetcode": "https://leetcode.com/problems",
    "cses": "https://cses.fi/problemset/task",
    "codeforces": "https://codeforces.com/problemset/problem",
    "project euler": "https://projecteuler.net/problem=",
    "dmoj": "https://dmoj.ca/problem"
}



from setuptools import find_packages

from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


def setup_package():
    metadata = dict(
        name="sorcerer",
        version="0.11",
        packages=find_packages(),
        install_requires=["Click>=7.0"],
        author="Liam Scalzulli",
        author_email="liamscalzulli@gmail.com",
        description="Programming problem solution README table generator",
        long_description=long_description,
        long_description_content_type='text/markdown',
        keywords="programming competitive tables markdown readmes",
        url="https://github.com/terror/sorcerer",
        project_urls={
            "Source Code": "https://github.com/terror/sorcerer",
        },
        classifiers=[
            "Development Status :: 3 - Alpha",
            "License :: OSI Approved :: MIT License",
            "Programming Language :: Python",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
            "Programming Language :: Python :: 3 :: Only",
        ],
        entry_points={"console_scripts": ["sorcerer = sorcerer.cli:cli"]},
        python_requires=">=3.7",
    )

    try:
        from setuptools import setup
    except ImportError:
        from distutils.core import setup
    setup(**metadata)


if __name__ == "__main__":
    setup_package()



languages = {
    ".py": "Python",
    ".cpp": "C++",
    ".hs": "Haskell",
    ".js": "Javascript",
    ".rs": "Rust",
    ".c": "C",
    ".ts": "Typescript",
    ".cs": "C#",
    ".go": "Go",
    ".java": "Java",
    ".ml": "OCaml",
    ".php": "PHP",
    ".rb": "Ruby",
    ".m": "Objective C"
}
